# -*- coding: utf-8 -*-

from upload.node.node import Node, AifConverterParameters, SmagConverterParameters, QueueInserterConfig, QueueParameters, SmagUkm4ConverterParameters, OracleConverterParameters, ShtrihConverterParameters, UsersConverterParameters, AtolConverterParameters, CrystalConverterParameters
import registry
import tempfile
import os, sys
from common import macros, logger, config

registryPath = '/linuxcash/cash/data/cash.reg'
reg = registry.Registry(registryPath)
log = logger.Logger().getLoggerUpload("main")
badDictsDirectory = '%s/exchangers/badDicts' % tempfile.gettempdir()

shopCode = str(reg.shopcode())
cashCode = str(reg.cashcode())

def readStandartParams(section):
    flags = [ macros.applyMacros(fl.strip(), log, shopcode = shopCode, cashcode = cashCode) for fl in config.getString(section, 'flags', '/tmp/dict/aif/pos*.flz').split(',') ]
    data = [ macros.applyMacros(d.strip(), log, shopcode = shopCode, cashcode = cashCode) for d in config.getString(section, 'data', '/tmp/dict/aif/pos*.aif').split(',') ]
    errorDest = macros.applyMacros(config.getString(section, 'errorDest', '').strip(), log, shopcode = shopCode, cashcode = cashCode)
    return flags, data, errorDest

def getConverter(converterName):
    converter = None
    section = 'upload.converter.%s' % converterName
    flags, data, errorDest = readStandartParams(section)

    if converterName == 'aif':
        hotKeyByBarcode = config.getBool(section, 'hotKeyByBarcode', False)
        converter = AifConverterParameters(flags = flags, data = data, errorDest = errorDest, hotKeyByBarcode = hotKeyByBarcode)
    elif (converterName == 'shtrih'):
        delimiter = config.getString(section, 'delimiter', ';')
        cashierName = config.getBool(section, 'cashierName', False)
        hotKeyByBarcode = config.getBool(section, 'hotKeyByBarcode', False)
        converter = ShtrihConverterParameters(flags = flags, data = data, errorDest = errorDest,
                delimiter = delimiter, cashierName = cashierName, hotKeyByBarcode = hotKeyByBarcode)
    elif (converterName == 'atol'):
        converter = AtolConverterParameters(flags = flags, data = data, errorDest = errorDest)
    elif (converterName == 'oracle'):
        converter = OracleConverterParameters(flags = flags, data = data, errorDest = errorDest)
    elif (converterName in ['smag-ukm', 'smag-ukm4']):
        charset = config.getString(section, 'charset', '866')
        format = config.getString(section, 'format', 'dat')
        defaultRoleCode = config.getInt(section, 'defaultRoleCode', 3)
        additionallyRoleCode = config.getInt(section, 'additionallyRoleCode', 2)
        usersWithAdditionallyRole = config.getString(section, 'usersWithAdditionallyRole', '')
        relationRolesBy = config.getString(section, 'relationRolesBy', 'name')
        substringInAdminName = config.getString(section, 'substringInAdminName', 'Администратор')
        catalogBindCode = config.getString(section, 'catalogBindCode', '')
        groupBindCode = config.getString(section, 'groupBindCode', '')

        if converterName == 'smag-ukm':
            converter = SmagConverterParameters(flags = flags, data = data, errorDest = errorDest, charset = charset, format = format,
                                                      defaultRoleCode = defaultRoleCode, additionallyRoleCode = additionallyRoleCode,
                                                      usersWithAdditionallyRole = usersWithAdditionallyRole, relationRolesBy = relationRolesBy,
                                                      substringInAdminName = substringInAdminName,
                                                      catalogBindCode=catalogBindCode,
                                                      groupBindCode=groupBindCode)
        else:
            discount = config.getString(section, 'discount', 'modern')
            usersWithAdditionallyRole = usersWithAdditionallyRole.split(',')
            converter = SmagUkm4ConverterParameters(flags=flags,
                                                          data=data,
                                                          errorDest=errorDest,
                                                          charset=charset,
                                                          format=format,
                                                          discount=discount,
                                                          defaultRoleCode=defaultRoleCode,
                                                          additionallyRoleCode=additionallyRoleCode,
                                                          usersWithAdditionallyRole=usersWithAdditionallyRole,
                                                          relationRolesBy=relationRolesBy,
                                                          substringInAdminName=substringInAdminName,
                                                          catalogBindCode=catalogBindCode,
                                                          groupBindCode=groupBindCode)
    elif (converterName == 'users'):
        converter = UsersConverterParameters(flags = flags, data = data, errorDest = errorDest)
    elif (converterName == 'crystal'):
        visualVerifySG = config.getString(section, 'visualverifysg', '1,2,3')
        requireQuantityManualSG = config.getString(section, 'requirequantitymanualsg', '4,5,6')
        measureMapper = unicode(config.getString(section, 'measuremap', 'шт-1,1007-2'), 'utf-8')
        converter = CrystalConverterParameters(flags = flags, data = data, errorDest = errorDest,
                                                    visualVerifySG=visualVerifySG, 
                                                    requireQuantityManualSG=requireQuantityManualSG,
                                                    measureMapper=measureMapper)
    return converter

def getNodes():
    log.info(u"Получение ноды")
    configFileName = os.path.normpath(sys.path[0] + "/../config/cash/exchangers.ini")
    if not config.read(configFileName):
        log.error(u"Файл настроек '%s' не найден" % configFileName)
        sys.exit(1)

    if not shopCode or shopCode == '':
        log.error(u"Ошибка получения кода кассы из %s" % registryPath)
        sys.exit(1)

    queue_host = config.getString('QUEUE', 'host', 'localhost') 
    queue_port = config.getInt('QUEUE', 'port', 5672)
    queue_user = config.getString('QUEUE', 'user', 'guest')
    queue_password = config.getString('QUEUE', 'password', 'guest')
    aifObjectsCountInMessage = config.getInt('QUEUE', 'aifObjectsCountInMessage', 200)

    badDictsDirectory = macros.applyMacros(config.getString('UPLOAD', 'badDictsDirectory', '/linuxcash/cash/data/backup/badDicts').strip(), log, shopcode = shopCode, cashcode = cashCode)
    convertersInConf = config.getString('UPLOAD', 'converter', 'aif').split(',')
    converters = []
    for converterName in convertersInConf:
        converterName = converterName.strip().lower()
        converter = getConverter(converterName)
        if converter:
            converters.append(converter)
            log.info(u'Конвертер - \'%s\'' % converterName)
            log.info(u'Файлы данных - \'%s\'' % converter.data)
            log.info(u'Файлы флагов - \'%s\'' % converter.flags)
            log.info(u'Файл информации об ошибках - \'%s\'' % converter.errorDest)

    nodes = []
    nodes.append(
                Node(id = shopCode,
                    backup_dir = macros.applyMacros(config.getString('UPLOAD', 'backupDir', '/linuxcash/cash/data/backup/dicts'), log, shopcode = shopCode, cashcode = cashCode), 
                    converters = converters, 
                    pack_size = aifObjectsCountInMessage, 
                    inserter = QueueInserterConfig(queue = QueueParameters(host = queue_host, port = queue_port, username=queue_user, password=queue_password, queue='artix.loaddict.cash')),
                    badDictsDirectory = badDictsDirectory)
                )
    log.info(u"Получение ноды завершено")
    return nodes
